@@ -8,16 +8,15 @@ import numpy as np
 import Bio.SeqIO
 
 from midas2.common.argparser import add_subcommand
-from midas2.common.utils import tsprint, num_physical_cores, InputStream, OutputStream, select_from_tsv
+from midas2.common.utils import tsprint, num_physical_cores, InputStream, OutputStream, select_from_tsv, args_string
 from midas2.models.midasdb import MIDAS_DB
 from midas2.models.sample import Sample
-from midas2.params.schemas import BLAST_M8_SCHEMA, MARKER_INFO_SCHEMA, species_profile_schema, species_marker_profile_schema, format_data, DECIMALS6
+from midas2.params.schemas import BLAST_M8_SCHEMA, MARKER_INFO_SCHEMA, species_profile_schema, format_data, DECIMALS6
 from midas2.params.inputs import MIDASDB_NAMES
 
 
 DEFAULT_WORD_SIZE = 28
 DEFAULT_ALN_COV = 0.75
-DEFAULT_ALN_MAPID = 94.0
 DEFAULT_MARKER_READS = 2
 DEFAULT_MARKER_COVERED = 2
 
@@ -50,7 +49,7 @@ def register_args(main_func):
                            dest='midasdb_dir',
                            type=str,
                            default="midasdb",
-                           help=f"Local MIDAS Database path mirroing S3.")
+                           help=f"Path to local MIDAS Database.")
 
     subparser.add_argument('--word_size',
                            dest='word_size',
@@ -188,16 +187,14 @@ def read_markers_info(fasta_file, map_file, genes_that_are_marker_fp):
     # Extract the mapping of <marker_id, genome_id, species_id> from the phyeco.map file
     markers_info = defaultdict(dict)
     markers_length = defaultdict(lambda: defaultdict(int))
-    markers_gene_list = defaultdict(lambda: defaultdict(list))
-    filter_cmd = f"grep -Fwf {genes_that_are_marker_fp}"
 
+    filter_cmd = f"grep -Fwf {genes_that_are_marker_fp}"
     with InputStream(map_file, filter_cmd) as stream:
         for r in select_from_tsv(stream, schema=MARKER_INFO_SCHEMA, result_structure=dict):
             markers_info[r["gene_id"]] = r
             markers_length[r['species_id']][r['marker_id']] += int(r["gene_length"])
-            markers_gene_list[r['species_id']][r['marker_id']].append(r["gene_id"])
         stream.ignore_errors()
-    return markers_info, markers_length, markers_gene_list
+    return markers_info, markers_length
 
 
 def find_best_hits(m8_file, markers_info, marker_cutoffs, args):
@@ -213,7 +210,6 @@ def find_best_hits(m8_file, markers_info, marker_cutoffs, args):
             cutoff = args.aln_mapid if args.aln_mapid is not None else cutoff
             if aln['pid'] < cutoff:
                 continue
-
             if query_coverage(aln) < args.aln_cov: # filter local alignments
                 continue
 
@@ -260,7 +256,6 @@ def assign_unique(alns, markers_info, args):
     unique_alns = defaultdict(lambda: defaultdict(dict))
     unique_counts = defaultdict(int)
     non_unique_counts = 0
-
     unique_reads = defaultdict(lambda: defaultdict(list))
     for aln in alns:
         if len(aln) == 1:
@@ -354,16 +349,19 @@ def merge_counts(unique_alns, ambiguous_alns, unique_covered_markers, ambiguous_
         amb_covered = len(lom_amb)
         total_covered = len(set(lom_uniq + lom_amb))
 
-        species_covered_markers[spid] = {"total_covered":total_covered, "unique_covered":uniq_covered, "ambiguous_covered":amb_covered, "total_marker_counts": len(marker_dict.keys())}
-
+        species_covered_markers[spid] = {
+            "total_covered":total_covered,
+            "unique_covered":uniq_covered,
+            "ambiguous_covered":amb_covered,
+            "total_marker_counts": len(marker_dict.keys())
+        }
     return species_alns, species_covered_markers
 
 
-def normalize_counts(species_alns, species_covered_markers, markers_length, markers_gene_list):
+def normalize_counts(species_alns, species_covered_markers, markers_length):
     """ Normalize counts by gene length and sum contrain """
 
-    species_abundance = defaultdict(lambda: defaultdict(int)) # indexed by <species_id>
-    markers_abundance = defaultdict(lambda: defaultdict(dict)) # indexed by <species_id, marker_id>
+    sp_abun = defaultdict(lambda: defaultdict(int)) # indexed by <species_id>
     for spid, sp_mkdict in species_alns.items():
         lomc = []
         for mkid, mkdict in sp_mkdict.items():
@@ -376,62 +374,42 @@ def normalize_counts(species_alns, species_covered_markers, markers_length, mark
             lomc.append(mkcov)
 
             # Accumulate information across markers per species
-            species_abundance[spid]["read_counts"] += readcounts
-            species_abundance[spid]["total_bps"] += aln_bps
-            species_abundance[spid]["total_marker_length"] += mklength
-
-            # Record information for <species_id, marker_id>
-            markers_abundance[spid][mkid]["gene_id"] = ",".join(markers_gene_list[spid][mkid])
-            markers_abundance[spid][mkid]["length"] = mklength
-            markers_abundance[spid][mkid]["coverage"] = mkcov
-            markers_abundance[spid][mkid]["total_reads"] = mkdict["unique"] + mkdict["ambiguous"]
-            markers_abundance[spid][mkid]["uniq_reads"] = mkdict["unique"]
-            markers_abundance[spid][mkid]["ambi_reads"] = mkdict["ambiguous"]
-            markers_abundance[spid][mkid]["total_alnbps"] = mkdict["unique_bps"] + mkdict["ambiguous_bps"]
-            markers_abundance[spid][mkid]["uniq_alnbps"] = mkdict["unique_bps"]
-            markers_abundance[spid][mkid]["ambi_alnbps"] = mkdict["ambiguous_bps"]
+            sp_abun[spid]["read_counts"] += readcounts
+            sp_abun[spid]["total_bps"] += aln_bps
+            sp_abun[spid]["total_marker_length"] += mklength
 
         # Second round of collecting information per species
-        species_abundance[spid]["coverage"] = species_abundance[spid]["total_bps"] / species_abundance[spid]["total_marker_length"]
-        species_abundance[spid]["median_coverage"] = np.median(lomc)
+        sp_abun[spid]["coverage"] = sp_abun[spid]["total_bps"] / sp_abun[spid]["total_marker_length"]
+        sp_abun[spid]["median_coverage"] = np.median(lomc)
 
-        species_abundance[spid]["total_covered"] = species_covered_markers[spid]["total_covered"]
-        species_abundance[spid]["unique_covered"] = species_covered_markers[spid]["unique_covered"]
-        species_abundance[spid]["ambiguous_covered"] = species_covered_markers[spid]["ambiguous_covered"]
-        species_abundance[spid]["total_marker_counts"] = species_covered_markers[spid]["total_marker_counts"]
-        species_abundance[spid]["unique_fraction_covered"] = species_abundance[spid]["unique_covered"] / species_abundance[spid]["total_marker_counts"]
+        sp_abun[spid]["total_covered"] = species_covered_markers[spid]["total_covered"]
+        sp_abun[spid]["unique_covered"] = species_covered_markers[spid]["unique_covered"]
+        sp_abun[spid]["ambiguous_covered"] = species_covered_markers[spid]["ambiguous_covered"]
+        sp_abun[spid]["total_marker_counts"] = species_covered_markers[spid]["total_marker_counts"]
+        sp_abun[spid]["unique_fraction_covered"] = sp_abun[spid]["unique_covered"] / sp_abun[spid]["total_marker_counts"]
 
     # Third round: compute relative abundance based on vertical coverage: only among detected species.
-    sample_coverage = sum(_['coverage'] for _ in species_abundance.values())
-    for r in species_abundance.values():
+    sample_coverage = sum(_['coverage'] for _ in sp_abun.values())
+    for r in sp_abun.values():
         r["relative_abundance"] = r["coverage"] / sample_coverage if sample_coverage > 0 else 0.0
 
     tsprint(f"  total marker-gene coverage {sample_coverage:.3f}")
-    return species_abundance, markers_abundance
+    return sp_abun
 
 
-def write_abundance(species_path, markers_path, species_abundance, markers_abundance):
+def write_abundance(species_path, sp_abun):
     """ Write species results to specified output file """
     # Sort the species by median_coverage
-    output_order = sorted(species_abundance.keys(), key=lambda sid: species_abundance[sid]['median_coverage'], reverse=True)
+    output_order = sorted(sp_abun.keys(), key=lambda sid: sp_abun[sid]['median_coverage'], reverse=True)
 
     with OutputStream(species_path) as outfile:
         outfile.write('\t'.join(species_profile_schema.keys()) + '\n')
         for species_id in output_order:
-            r = species_abundance[species_id]
-            record = [species_id, r['read_counts'], r['median_coverage'], r['coverage'], r['relative_abundance'],
-                      r['total_covered'], r['unique_covered'], r['ambiguous_covered'],
-                      r['total_marker_counts'], r['unique_fraction_covered'], r['total_marker_length']]
+            r = sp_abun[species_id]
+            record = [species_id, r['read_counts'], r['median_coverage'], r['coverage'],
+                      r['relative_abundance'], r['unique_fraction_covered']]
             outfile.write("\t".join(map(format_data, record, repeat(DECIMALS6, len(record)))) + "\n")
 
-    with OutputStream(markers_path) as outfile:
-        outfile.write('\t'.join(species_marker_profile_schema.keys()) + '\n')
-        for species_id in output_order:
-            for mid, md in markers_abundance[species_id].items():
-                record = [species_id, mid, md['length'], md["gene_id"], md['total_reads'], md['total_alnbps'],
-                          md['coverage'], md['uniq_reads'], md['ambi_reads'], md['uniq_alnbps'], md['ambi_alnbps']]
-                outfile.write("\t".join(map(format_data, record, repeat(DECIMALS6, len(record)))) + "\n")
-
 
 def run_species(args):
 
@@ -439,15 +417,17 @@ def run_species(args):
         sample = Sample(args.sample_name, args.midas_outdir, "species")
         sample.create_dirs(["outdir", "tempdir"], args.debug)
 
+        with OutputStream(sample.get_target_layout("species_log")) as stream:
+            stream.write(f"Single sample abundant species profiling in subcommand {args.subcommand} with args\n{json.dumps(args_string(args), indent=4)}\n")
+
         tsprint(f"MIDAS2::fetch_midasdb_files::start")
         midas_db = MIDAS_DB(os.path.abspath(args.midasdb_dir), args.midasdb_name)
-
+        midas_db.fetch_files("markerdb")
+        midas_db.fetch_files("markerdb_models")
         marker_db_files = midas_db.fetch_files("marker_db")
-        marker_db_hmm_cutoffs = midas_db.fetch_files("marker_db_hmm_cutoffs")
-
         tsprint(f"MIDAS2::fetch_midasdb_files::finish")
 
-        with InputStream(marker_db_hmm_cutoffs) as cutoff_params:
+        with InputStream(midas_db.get_target_layout("marker_db_hmm_cutoffs", False)) as cutoff_params:
             marker_cutoffs = dict(select_from_tsv(cutoff_params, selected_columns={"marker_id": str, "marker_cutoff": float}))
 
         # Align reads to marker-genes database
@@ -461,7 +441,7 @@ def run_species(args):
 
         tsprint(f"MIDAS2::read in marker information::start")
         genes_that_are_marker_fp = sample.get_target_layout("species_marker_genes")
-        markers_info, markers_length, markers_gene_list = read_markers_info(marker_db_files["fa"], marker_db_files["map"], genes_that_are_marker_fp)
+        markers_info, markers_length = read_markers_info(marker_db_files["fa"], marker_db_files["map"], genes_that_are_marker_fp)
         tsprint(f"MIDAS2::read in marker information::finish")
 
         # Classify reads
@@ -480,10 +460,12 @@ def run_species(args):
         # Estimate species abundance
         tsprint(f"MIDAS2::normalize_counts::start")
         species_alns, species_covered_markers = merge_counts(unique_alns, ambiguous_alns, unique_covered_markers, ambiguous_covered_markers, markers_length)
-        species_abundance, markers_abundance = normalize_counts(species_alns, species_covered_markers, markers_length, markers_gene_list)
+        species_abundance = normalize_counts(species_alns, species_covered_markers, markers_length)
         tsprint(f"MIDAS2::normalize_counts::finish")
+        write_abundance(sample.get_target_layout("species_summary"), species_abundance)
 
-        write_abundance(sample.get_target_layout("species_summary"), sample.get_target_layout("markers_summary"), species_abundance, markers_abundance)
+        if not args.debug:
+            sample.remove_dirs(["tempdir"])
 
     except Exception as error:
         if not args.debug:
@@ -494,5 +476,5 @@ def run_species(args):
 
 @register_args
 def main(args):
-    tsprint(f"Species abundance estimation in subcommand {args.subcommand} with args\n{json.dumps(vars(args), indent=4)}")
+    tsprint(f"Single sample abundant species profiling in subcommand {args.subcommand} with args\n{json.dumps(vars(args), indent=4)}")
     run_species(args)
